/* menu.c */

#include "mac/quickdraw.h"
#include "mac/osintf.h"
#include "mac/toolintf.h"
#include "othello.h"

#define LASTMENU	6	/* number of menus			*/
#define APPLEMENU	1	/* menu ID for desk accessory menu	*/
#define GAMEMENU	2	/* menu ID for Game menu		*/
#define MOVEMENU	3	/* menu ID for Move menu		*/
#define PLAYERMENU	4	/* menu ID for Players menu		*/
#define SKILLMENU	5	/* menu ID for Skill menu		*/
#define TRACEMENU	6	/* menu ID for Trace menu		*/

#define INEWGAME	1	/* items in the Game menu		*/
#define IOLDGAME	2
#define ISAVEGAME	3
#define ISTOP		4
#define IQUIT		6

#define IUNDO		1	/* items in the Move menu		*/
#define IREDO		2
#define IREPEAT		3

#define IWHITE		1	/* items in the Players menu		*/
#define IBLACK		2

#define ILEVEL0		1	/* items in the Skill menu		*/
#define ILEVEL1		2
#define ILEVEL2		3
#define ILEVEL3		4
#define ILEVEL4		5
#define ILEVEL5		6
#define IINCSKILL	8
#define IDECSKILL	9

#define ITRACEOFF	1	/* items in the Trace menu		*/
#define ITRACEON	2
#define IVISUALTR	3
#define IVERBOSETR	4

#define  ABOUTOTHELLOID 1	/* Resource ID of the string */

MenuHandle myMenus[LASTMENU + 1];	/* our menus */

/* doCommand routines */
void	HandApple();
void	HandGame(), HandMove();
void	HandPlayer(), HandSkill(), HandTrace();

SetUpMenus()
{
	int i;
	/* use the fact that our menu ID's start at 1 */
	for (i = 1; i <= LASTMENU; i++)	/* get all my menus in */
		myMenus[i] = GetMenu(i);

	/* pull in all desk accessories */
	AddResMenu (myMenus[APPLEMENU], "DRVR");

	for (i = 1; i <= LASTMENU; i++)
		/* insert menus; 0 => put at end */
		InsertMenu (myMenus[i], 0);
	DrawMenuBar();
	CheckItem(myMenus[SKILLMENU], skill + ILEVEL0, TRUE);
	CheckItem(myMenus[TRACEMENU], trace + ITRACEOFF, TRUE);
}

/*
handle a command given through a menu selection
############################   DoCommand   ##############################

   We carry out the command indicated by mResult.
   If it was Quit, we return true, else false.  Since the menu was
   highlighted by MenuSelect, we must finish by unhighlighting it
   to indicate we're done.
*/
void	doCommand (mresult)
long    mresult;

{
	int themenu,theitem;

	themenu = HiWord (mresult); /* get the menu selected */
	theitem = LoWord (mresult); /* ... and the item of that menu */
	switch (themenu) {
		case 0: 
			break;	/* user made no selection; do nothing */
		case APPLEMENU: 
			HandApple(theitem);
			break;

		case GAMEMENU: 
			HandGame(theitem);
			break;

		case MOVEMENU:
			HandMove(theitem);
			break;
		
		case PLAYERMENU:
			HandPlayer(theitem);
			break;
		
		case SKILLMENU:
			HandSkill(theitem);
			break;
		
		case TRACEMENU:
			HandTrace(theitem);
			break;
    }				/* menu case */

    HiliteMenu (0);		/* turn off hilighting on the menu just
				   used */
}				/* DoCommand */


/* 
  It's important not to pass Report a de-referenced handle; if Report were
  in another segment, loading it could cause a memory compaction; the
  de-referenced handle could become invalid.  Watch out for this and
  similar nasties everywhere in your program.  See the Memory Manager and
  the Segment Loader.
*/
void
HandApple(theitem)
int theitem;
{
	GrafPtr saveport;	/* where to save current port */
	Handle astr;
	Handle myhandle;
	char name[255];
	int refnum;

	if (theitem == 1) {	/* get string, and tell about Skel */
		/* WARNING! getstring returns handle to Pascal-style */
		/* string! - WHJ 3/10/85 */
		/* isapstr() returns the pointer with bit 24 set, so	*/
		/* that QuickDraw routines in ROM will know not to	*/
		/* convert it from C format. - SBM 6/15/85		*/
		astr = GetString(ABOUTOTHELLOID);
		report(isapstr(*astr));
		return;
	}
	/* run a desk accesory (assume in range if selected */
	/* make sure our port is preserved */
	GetPort(&saveport);
	GetItem(myMenus[APPLEMENU], theitem, name);
					/* get name */
	SetResLoad(FALSE);		/* try not bringing into memory */
	myhandle=(Handle)NewHandle(SizeResource(GetNamedResource('DRVR', name))+3072);
	SetResLoad(TRUE);		/* well, okay, now go ahead */
	if (myhandle == NIL) {
		SysBeep(1);
	}
	else {
		DisposHandle(myhandle);	/* was just checking! */
		refnum = OpenDeskAcc(DAname(name)); /* run desk accessory */
		SetPort(saveport);
	}
}


/*
 * Handle game menu items
 *
*/
void
HandGame(theitem)
int theitem;
{
	switch (theitem) {
		case INEWGAME:
			NewGame();
		    	break;
		case IOLDGAME:
		case ISAVEGAME:
			break;
		case ISTOP:
			StopThinking = TRUE;
			break;
		case IQUIT: 
			ExitToShell();
	}			/* fileMenu case */
}



void
HandMove(theitem)
int theitem;
{
	switch (theitem) {
		case IUNDO:
		case IREDO:
		case IREPEAT:
			UndoMove();
			break;
	}
}



/*
 * Handle Player menu items
 *
*/
void
HandPlayer(theitem)
int theitem;
{
	switch (theitem) {
		case IWHITE:
			player[stoneWhite] = !player[stoneWhite];
		    	break;
		case IBLACK:
			player[stoneBlack] = !player[stoneBlack];
			break;
	}
	UpdateInfo();
}



void
HandSkill(theitem)
int theitem;
{
	int	i;

	switch (theitem) {
		case ILEVEL0:
		case ILEVEL1:
		case ILEVEL2:
		case ILEVEL3:
		case ILEVEL4:
		case ILEVEL5:
			skill = theitem - ILEVEL0;
			break;
		case IINCSKILL:
			++skill;
			break;
		case IDECSKILL:
			if (skill > 0)
				--skill;
			break;
	}
	for (i = ILEVEL0; i <= ILEVEL5; ++i)
		CheckItem(myMenus[SKILLMENU], i, (skill == i - ILEVEL0));
	UpdateInfo();
}



void
HandTrace(theitem)
int	theitem;
{
	int	i;

	switch (theitem) {
	case ITRACEOFF:
		strcpy(message, "Trace off");
		break;
	case ITRACEON:
		strcpy(message, "Trace on");
		break;
	case IVISUALTR:
		strcpy(message, "Visual trace");
		break;
	case IVERBOSETR:
		strcpy(message, "Verbose trace");
		break;
	}
	trace = theitem - ITRACEOFF;
	for (i = ITRACEOFF; i <= IVERBOSETR; ++i)
		CheckItem(myMenus[TRACEMENU], i, (i == theitem));
	UpdateInfo();
}
