/* move.c - ChooseMove, EvalMove, analyze, RestoreBoard, SortMoves */

#include "mac/quickdraw.h"
#include "mac/osintf.h"
#include "mac/toolintf.h"
#include "othello.h"

unsigned short Random();
short EvalMove(), analyze();

short position[BOARDSIZE+2][BOARDSIZE+2] =
	{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	   0, 20,  3,  4,  4,  4,  4,  3, 20,  0,
	   0,  3, -7, -1, -1, -1, -1, -7,  3,  0,
	   0,  4, -1,  0,  0,  0,  0, -1,  4,  0,
	   0,  4, -1,  0,  0,  0,  0, -1,  4,  0,
	   0,  4, -1,  0,  0,  0,  0, -1,  4,  0,
	   0,  4, -1,  0,  0,  0,  0, -1,  4,  0,
	   0,  3, -7, -1, -1, -1, -1, -7,  3,  0,
	   0, 20,  3,  4,  4,  4,  4,  3, 20,  0,
	   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 };

BoardArray	WorkBoard;
int	OrigLookAhead;
int	skill = 0;
int	StopThinking = FALSE;
int	nTicks[nTimed] = {0, 0, 0, 0, 0};



ChooseMove(move, board)
MoveRecord	*move;
BoardArray	board;
{
	MoveList	moves;
	int		nMoves, i;
	int		ticks, other;
	short		analysis;
	register int	row, col;
	
	for (i = 0; i < nTimed; ++i)
		nTicks[i] = 0;
	ticks = TickCount();
	StopThinking = FALSE;
	analysis = 0;
	for (row = 1; row <= BOARDSIZE; ++row)
	    for (col = 1; col <= BOARDSIZE; ++col) {
		board[row][col] &= stoneColor;
		WorkBoard[row][col] = board[row][col];
		if ((board[row][col] & stoneColor) == curColor)
			analysis += position[row][col];
		else if ((board[row][col] & stoneColor) == opposite(curColor))
			analysis -= position[row][col];
	    }
#ifdef UNDEF
	nMoves = FindMoves(opposite(curColor), WorkBoard, moves, ALL);
	for (i = 0; i < nMoves; ++i)
		moves[i].value = EvalMove(moves[i], WorkBoard,
			opposite(curColor), skill - 1);
	SortMoves(moves, nMoves);
	for (i = 0; moves[i].value == moves[0].value; ++i)
		WorkBoard[moves[i].row][moves[i].col] |= threat;
#endif
	nMoves = FindMoves(curColor, WorkBoard, moves, ALL);
	if (nStones[stoneEmpty] > EndGame)
		OrigLookAhead = skill;
	else
		OrigLookAhead = EndGame;
	for (i = 0; i < nMoves; ++i)
		moves[i].value = EvalMove(moves[i], WorkBoard, curColor,
					OrigLookAhead);
	SortMoves(moves, nMoves);
	while (moves[--nMoves].value < moves[0].value)
		;
	++nMoves;
	*move = moves[Random() % nMoves];
	analysis += move->value;
	ticks = TickCount() - ticks;
	if (!trace || StopThinking)
		ClearMessages();
	else {
		sprintf(note1, "Analysis = %d (%d)", analysis, move->value);
		sprintf(note2, "fm %d%% df %d%% a %d%% r %d%%",
			100 * nTicks[pFindMoves]	/ ticks,
			100 * nTicks[pDoFlips]		/ ticks,
			100 * nTicks[pAnalyze]		/ ticks,
			100 * nTicks[pRestoreBoard]	/ ticks);
		other = ticks;
		for (i = 0; i < nTimed; ++i)
			other -= nTicks[i];
		sprintf(note3, "sort %d%% other %d%%",
			100 * nTicks[pSortMoves]	/ ticks,
			100 * other			/ ticks);
	}
}

short EvalMove(move, board, color, LookAhead)
MoveRecord	move;
BoardArray	board;
int		color, LookAhead;
{
	register int	row, col, i;
	MoveList	flips, moves, oflips;
	int		nFlips, nMoves, nToTry, nOFlips;
	short		value, best, factor, analysis;
	int		OrigTrace;

	DoEvent();			/* Respond to the user.		*/
	OrigTrace = trace;
	analysis = analyze(move, color, flips, &nFlips, board, OrigTrace);
	if (LookAhead <= 0 || LookAhead <= OrigLookAhead - MaxLookAhead ||
		nStones[stoneEmpty] <= 0 || StopThinking) {
		RestoreBoard(board, move, flips, nFlips, OrigTrace);
		return(analysis);
	}
	best = minValue;
	if ((nMoves=FindMoves(opposite(color), board, moves, RESTRICT)) > 0 ||
		(nMoves=FindMoves(opposite(color), board, moves, ALL)) > 0) {
		/* Subtract opponent's best choice */
		color = opposite(color);
		factor = -1;
	}
	else if ((nMoves = FindMoves(color, board, moves, ALL)) > 0)
		/* Opponent can't move; evaluate my further choices */
		factor = 1;
	else {
		/* Game is over (no one can move); count up stones */
		value = 0;
		for (row = 1; row <= BOARDSIZE; ++row)
		    for (col = 1; col <= BOARDSIZE; ++col)
			switch (board[row][col] & stoneColor) {
			case stoneWhite:
				++value;
				break;
			case stoneBlack:
				--value;
				break;
			}
		if (color == stoneBlack)
			value = -value;
		RestoreBoard(board, move, flips, nFlips, OrigTrace);
		return(100 * value);
	}
	for (i = 0; i < nMoves; ++i) {
		moves[i].value = analyze(moves[i], color, oflips, &nOFlips,
			board, OrigTrace);
		RestoreBoard(board, moves[i], oflips, nOFlips, OrigTrace);
	}
	SortMoves(moves, nMoves);
	if (LookAhead < skill)
		nToTry = nMoves * LookAhead*LookAhead/(skill*skill) + 1;
	else
		nToTry = nMoves;
	for (i = 0; i < nToTry; ++i) {
		if (moves[i].value < 0 && moves[i].value < moves[0].value)
			break;
		if ((value = EvalMove(moves[i], board, color, LookAhead-1)) <
			best - 10)
			break;
		if (value > best)
			best = value;
	}
	RestoreBoard(board, move, flips, nFlips, OrigTrace);
	if (skill <= 1)
		return(analysis + factor*best);
	else	/* Leave the opponent as few choices as possible */
		return(analysis + factor*(best + nMoves/2));
}



short analyze(move, color, flips, nFlips, board, trace)
MoveRecord	move;
MoveList	flips;
int		*nFlips;
BoardArray	board;
int		trace;
{
	int		opponent;
	register int	i, drow, dcol, nNeighbors;
	short		value;
	int		ticks;
	
	if (trace > 1)
		PlaceStone(move.row, move.col, color);
	board[move.row][move.col] =
	board[move.row][move.col] & ~stoneColor | color;
	board[move.row][move.col] |= flipped;
	*nFlips = DoFlips(color, move, board, flips, INVISIBLE);
	ticks = TickCount();
	opponent = opposite(board[move.row][move.col]);
	if ((value = position[move.row][move.col]) < -2)
		value += value;		/* Placing as bad as flipping here */
	nNeighbors = -1;
	for (drow = -1; drow <= 1; ++drow)
	    for (dcol = -1; dcol <= 1; ++dcol)
		if (board[move.row+drow][move.col+dcol] != stoneEmpty)
			++nNeighbors;
	value += nNeighbors/2 - 3;	/* Keep stones bunched up */
	if ((move.row == 1 || move.row == BOARDSIZE) &&
		(board[move.row][move.col - 1] & stoneColor) == opponent &&
		(board[move.row][move.col + 1] & stoneColor) == opponent)
		value *= 4;
	else if ((move.col == 1 || move.col == BOARDSIZE) &&
		(board[move.row - 1][move.col] & stoneColor) == opponent &&
		(board[move.row + 1][move.col] & stoneColor) == opponent)
		value *= 4;
	for (i = 0; i < *nFlips; ++i) {
		nNeighbors = -1;
		for (drow = -1; drow <= 1; ++drow)
		    for (dcol = -1; dcol <= 1; ++dcol)
			if (board[flips[i].row+drow][flips[i].col+dcol] !=
				stoneEmpty)
				++nNeighbors;
		value += 2 * position[flips[i].row][flips[i].col] +
			nNeighbors/2 - 3;	/* Keep stones bunched up */
	}
	if (trace > 2) {
		sprintf(note1, "analyze = %d", value);
		display(30);
	}
	nTicks[pAnalyze] += TickCount() - ticks;
	return(value);
}

RestoreBoard(board, move, flips, nFlips, trace)
BoardArray	board;
MoveRecord	move;
MoveList	flips;
int		nFlips;
int		trace;
{
	register int	color;	/* color to change stones back to	*/
	register int	i;
	int		ticks;
	
	ticks = TickCount();
	color = opposite(board[move.row][move.col]);
	if (trace > 1)
		PlaceStone(move.row, move.col, stoneEmpty);
	board[move.row][move.col] &= ~(flipped | stoneColor);
	for (i = 0; i < nFlips; ++i) {
		if (trace > 1)
			PlaceStone(flips[i].row, flips[i].col, color);
		board[flips[i].row][flips[i].col] =
		board[flips[i].row][flips[i].col] & ~(flipped | stoneColor)
						  | color;

	}
	nTicks[pRestoreBoard] += TickCount() - ticks;
}



SortMoves(moves, nMoves)
MoveList	moves;
int		nMoves;
{
	register int	i, nSorted;
	int		max;
	MoveRecord	temp;
	int		ticks;
	
	ticks = TickCount();
	for (nSorted = 0; nSorted < nMoves; ++nSorted) {
		max = nSorted;
		for (i = nSorted + 1; i < nMoves; ++i)
			if (moves[i].value > moves[max].value)
				max = i;
		if (max != nSorted) {
			temp = moves[nSorted];
			moves[nSorted] = moves[max];
			moves[max] = temp;
		}
	}
	nTicks[pSortMoves] += TickCount() - ticks;
}
