/* othello.c */

/*
 * This game written for the Macintosh by Steven Munson.
 * Copyright (c) 1984, Steven Munson
 */

/* .h files needed by skeleton */

#include "mac/quickdraw.h"
#include "mac/osintf.h"
#include "mac/toolintf.h"
#include "othello.h"

/*    
    The following two constants are not defined in any .h file.  They
    are described in "Inside Macintosh" - Event Manager, pp. 14-15.
  Well, Sumac defines cmdkey at least.
*/
#define CHARCODEMASK 255  

#define WINDOWID 260		/* Resource ID for my window */

#define RPTBOXID 257		/* ID of our report dialog in resource file */
#define RPTTEXT 2		/* Item # of dialog's report text */

typedef long   *lomemptr;	/* a pointer to low memory locations */


/* our global vars */
struct QDVar qdvar;
GrafPtr ScreenPort;		/* a port for the whole screen */
WindowPtr myWindow;		/* our one window */
WindowRecord wRecord;		/* storage for window record */
Rect DragRect;			/* rect to drag within */
Rect GrowRect;			/* bounds for the growth of the windows */

int	trace = 0;
int	curColor = stoneWhite;
int	player[] = {0, Macintosh, Person, 0};		/* color -> player */
char	*playerName[] = {"Macintosh", "Person"};	/* player -> name */
int	nStones[] = {BOARDSIZE*BOARDSIZE, 0, 0, 0};	/* color -> n stones */
char	*colorName[] = {"Empty", "White", "Black", "Edge"};
int	GameOver = FALSE;
int	nClicks = 0;


/* declare to be paranoid */
/* DoEvent routines */
void doMouse();
void doAutoKey();
void doActivate();
void doUpdate();



/* from Bill Schilit, C.U. */
/*
 * Desk accessories have weird names that begin with a leading zero.
 * Since this fools the automatic C/Pascal string conversion stuff, we
 * have this little gem;  given a C string, it returns a Pascal string
 * with leading zero.  Luckily I know of no other dependencies on embedded
 * zeros in Macintosh strings.
 *
 * NB: causes problems with DA's that don't have leading zero's on their names
 * (e.g. with non apple DA's).
 */
 
char *
DAname(s)
char *s;
{
	static char ps[32];
	int i = 1;
	register char *cp,*dp;

	cp = s;
	dp = &ps[2];
	while ((*dp++ = *cp++))
		i++;
	ps[0] = i;
	return (isapstr(ps));
}


char *
pstrcpy(cStr, hPStr)
char	*cStr;
Handle	hPStr;
{
	register int	length;
	register char	*to, *from;
	
	to = cStr;
	length = **hPStr;
	from = *hPStr + 1;
	for ( ; length > 0; --length)
		*to++ = *from++;
	*to = '\0';
	return (cStr);
}



/* 
Print a string in dialog box
############################   Report   #################################

We put up a dialog box, show the string, and wait for user to hit OK.

n.b. for sumac, I believe you reference pascal strings at the start of
the string with the count right before it.  Not sure.  This would account
for the way this routine had to be called.  Also, it is probably best to 
do "rounding" (e.g. fill out to even # of bytes)

*/

report(reportstr)
char *reportstr;
{
	int     itemhit;	/* which Item was clicked on (only OK avail) */
	DialogPtr reportptr;

	/* 
		WARNING: paramtext assumes that the strings
		passed to it are Pascal-type, byte-encoded strings,
		instead of C-type, NULL-terminated strings.
		This works here for two reasons: First, getstring()
		returns a handle to the resource, which is in the
		proper format; and second, the Pascal and C style
		empty strings happen to look the same - WHJ 3/11/85

		Exercise: Modify Report so that it displays an
		internally generated string instead of a string
		obtained from the resource fork.  Be careful! This
		is pretty straightforward, but you can get stung!
	*/
	ParamText (reportstr, "", "", "");/* set text to display */
	reportptr = GetNewDialog (RPTBOXID, NULL, (long) - 1);

	/* get from Resource file; NIL => use heap storage;  */
	/* -1 => make dlg frontmost */

	/* carry out dialog; NIL => no FilterProc; return item Hit when done */
	ModalDialog (NULL, &itemhit);

	/* release storage and remove dialog from screen */
	DisposDialog (reportptr);
}


/*

SetUps for handling memory
##########################   SetUpMemory   ############################
   This very important set of initializations can be left out of the
   first versions of a program. We are making sure that memory is laid
   out as we desire, with adequate protection against running out of
   memory, bad handles, etc.
*/

SetUpMemory()
{
	lomemptr nilptr;	/* will have value NIL */
	lomemptr stackbaseptr;	/* points to current stack base */

	/* A GrowZone function to handles bad memory problems */
	/*  setgrowzone(&mygrowzone);     */


	/*  Place a longint -1 (an odd and therefore illegal address) */
	/* in the memory location that would be referenced by an */
	/* accidentally-NULL handle, so the error will be caught */
	/* at handle-reference time (as an Address error, ID=02) instead */
	/* of later on. */

	nilptr = NULL;
	*nilptr = -1;

	/* If you needed to use an Application heap limit other than the */
	/* default (which allows 8K for the stack), you'd set it here, */
	/* possible using this technique of explicitly specifying the */
	/*  maximum stack size and allocating the rest to the heap.  */
	/* Should be independent of memory size. */
	/* CurStackBase from Tlasm/sysequ.text */
	stackbaseptr = (lomemptr) 0x908;
	SetApplLimit((char *) (*stackbaseptr - MAXSTACKSIZE));


	/*  Expand the application heap zone to its maximum size, without */
	/* purging any purgeable resources.  This saves memory compactions */
	/* and heap expansions later. */
	MaxApplZone();

	/* get plenty of master pointers now; if we let the Memory Manager */
 	/* allocate them as needed, they'd form non-relocatable islands in */
	/* the heap.  */
	MoreMasters();
	MoreMasters();
	MoreMasters();

/* 
  Here you might install bulwarks against running out of memory
  unexpectedly.  One such (cheesy) technique is to here allocate
  a large handle, call it "CheeseBuf", which you can de-allocate
  in your GrowZone function, when you must obtain more memory to
  avoid a crash.  While de-allocated, the program could prevent
  the user from doing anything requiring memory, and tell him he
  must discard windows or some such memory freeing action. Each
  time he does so, the program can try to re-allocate CheeseBuf;
  if it succeeds, the user can go on doing memory-eating operations.
*/

}

/*
	
body of SetUp
############################   SetUp   ##############################

   Initialize our program.  It seems best to handle:
   Memory inits first, ToolBox inits second, then the program variables'
   inits. Note that the order of inits is important; see "Using the
   Dialog Manager" in the Dialog Mgr section.

*/

SetUp() 
{
	Rect screenrect;	/* screen size; could be machine-dependent */

	SetUpMemory();		/*  init memory layout and protection */

	/*  init QuickDraw, and everybody else */
	QD = &qdvar;
	InitGraf(&thePort);
	QD->randSeed = TickCount() ;
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);	/* NULL => no Restart proc; see Dialog Mgr */
				/* and System Error Handler */
	InitCursor();

	/* 
	    Init the system event mask, in case the previous program left
	    it in a bad state.  If you set it non-standard here, FIX IT
	    BEFORE EXITING, because the Finder (1.1g) does NOT set it.
        */
	SetEventMask(everyEvent - keyUpMask);/* standard setting */

	/* 
 	   Get the port which is the whole screen, to use when deactivating
	   our window.  This prevents the current grafPort pointer from
	   ever dangling.
	*/
	GetWMgrPort(&ScreenPort);	/* get whole screen port that window */
					/* manager uses... */
	SetPort(ScreenPort);	/* ... and start off with it */

	/* 
	  get window: use wRecord storage.  Port is set to that of the new
	  window.  GetNewWindow posts an update event for the new window, so it
	  will be redrawn right away.
	*/
	myWindow = GetNewWindow(WINDOWID, &wRecord, (long) - 1);
				/* -1 => frontmost window */

	/* set up dragRect; we can drag the window within it */
	/* don't assume screen size */
	screenrect.top = QD->screenBits.bounds.top;
	screenrect.left = QD->screenBits.bounds.left;
	screenrect.bottom = QD->screenBits.bounds.bottom;
	screenrect.right = QD->screenBits.bounds.right;

	/* set drag rect to avoid menu bar */
	SetRect(&DragRect,0,20,screenrect.right,screenrect.bottom);

	/* set up GrowRect, for limits on window growing */
	SetRect(&GrowRect,48,14,screenrect.right-7,screenrect.bottom-7);

	/* pull in and set up our menus */
	SetUpMenus();

	InitBoard();
}

/* 
Redraw my window
############################   DrawWindow   #############################

   We draw all the contents of our one window, myWindow.  Note that this
   must include scroll bar areas and the grow icon; the Window Manager
   will NOT handle those for us. Since there are no scroll bars, we
   must erase the region they would be in.  Echh.
*/

DrawWindow () {
	Rect arect;		/* rectangle to erase */

	/*  first, fill the window with white; fills scroll bars, too */
	EraseRect(&myWindow -> portRect);

	DrawBoard();
	DrawInfo();
}

/*

Update the contents of the given window
############################   UpdateWindow   ##########################

   This is our response to receipt of an update event for myWindow.
   Since the window is likely to be inactive, the current grafPort
   will be elsewhere.  We must change it for drawing, yet leave it
   as it was.
*/

UpdateWindow (awindow)
WindowPtr awindow;
{
	GrafPtr saveport;	/* to save and restore the old port */

	BeginUpdate(awindow);	/* reset ClipRgn etc to only redraw what's */
				/* necessary. */

	GetPort(&saveport);	/* don't trash the port; we might be */
				/* updating an inactive window */

	SetPort(awindow);	/* work in the specified window */
	DrawWindow();		/* redraw contents of window */

	SetPort(saveport);	/* all nice and tidy as before */
	EndUpdate(awindow);

}



/* 
Change the size of the given window
############################   ReSize   ###############################
  
  Called on a mouse-down in the grow box, this allows the user to change
  the size of the window.  We change the size, then
  claim that the whole of the window contents is no longer validly drawn,
  because we're too lazy to do so for just the scroll bar regions
  that are actually subject to change.  See the Window Manager.
*/

resize (awindow, downpt)
WindowPtr awindow;
Point * downpt;
{
	int     ww;
	int     wh;		/* new width and height of the sized window */
	long    newsize;	/* the new size */

	newsize = GrowWindow (awindow, downpt, &GrowRect);
	/* find new size */
	ww = LoWord (newsize);	/* find the width */
	wh = HiWord (newsize);	/* find the height */

	SizeWindow (awindow, ww, wh, 1);/* change to the new window size */

	/* place whole window into update region to be sure it all gets */
	/* updated. This is more than is strictly necessary, but a lot */
	/* easier than just invalidating the regions that actually may have */
	/* changed. */
	InvalRect (&awindow -> portRect);
}




/* 
the main loop that handles events
############################   MainEventLoop  ##########################
  
  Brace yourself: here's where the action is.  Most Mac programs just
  wait for events (as do we all), and then process them.  There are
  two sorts of events: those directly initiated by the user, like key
  presses and mouse-downs, and those consequent events posted by the
  Event Manager, like update and activate events.  The latter MUST be
  handled correctly and carefully.  In particular, it's important for
  all events to make sure that the event occurred in or for the window
  you expect -- it's possible to get events which are not for one of
  our windows, despite GetNextEvent's return value.  Similarly,
  be sure to check that the window it occured in is the active one,
  if it matters.
  
  A common mistake in handling update and activate events is in finding
  out which window they are for.  It is "(WindowPtr)event.message "
  that gives this information, NOT "whichWindow" (the WindowPtr returned
  by FindWindow).  The latter pointer merely tells you what window
  the mouse was in at the time the event was posted -- completely
  irrelevant for Update and Activate events.  Think it through carefully.
  
*/

DoEvent()
{

	EventRecord event;

	/* body of DoEvent */

	/* get next event and handle it appropriately */

	SystemTask();		/* handle desk accessories */

	/* Get event, return if no event */
	if (!GetNextEvent(everyEvent, &event))
		return;

	switch (event.what) {	/* handle each kind of event */
		case mouseDown: 
			doMouse(&event);
			break;	/* switch */

		case keyDown: 
		case autoKey: 
			doAutoKey(&event);
			break;	/* wasn't here before */

		case updateEvt: /* if it's for our window, update it */
			doUpdate(&event);
			break;

		case activateEvt: 
			doActivate(&event);
			break;
	}
}



/* find out what window the mouse went down in, and where in it */

void
doMouse(event)
EventRecord *event;
{
	WindowPtr whichWindow;	/* points to window of MouseDown */
	int windowcode;		/* what mouse was in when event posted */
	MoveRecord	move;

	windowcode = FindWindow (&event->where, &whichWindow);
	switch (windowcode) {
				/* handle mouse-down for each place */
		case inSysWindow: 
			/* handle the desk accessories */
			SystemClick (event, whichWindow);
			break;
		case inMenuBar:
			/* handle the command */
			doCommand (MenuSelect (&event->where));
		    	break;
		case inDrag:
			/* drag the window */
			DragWindow(whichWindow, &event->where, &DragRect);
			break;
		case inContent: 
			/* includes inGrow if window inactive. */
			/* Activate window */
			/* make sure it's for mine */
			if (whichWindow != myWindow)
				break;
			if (whichWindow != FrontWindow()) { /* activate it */
				SelectWindow (whichWindow);
				break;
			}
			if (GameOver)
				break;
			if (nClicks++ > 0)
				break;
			StopThinking = FALSE;
			GlobalToLocal(&event->where);
			if (player[curColor] == Macintosh)
				DoMove(move);
			else if (GetMove(&event->where, &move)) {
				DoMove(move);
				if (player[curColor] == Macintosh)
					DoMove(move);
			}
			while (--nClicks > 0) {	/* Handle mouse-ahead.	*/
				if (GameOver || StopThinking ||
					player[curColor] != Macintosh)
					break;
				UpdateInfo();
				DoMove(move);
			}
			nClicks = 0;
			UpdateInfo();
		    	break;
		case inGrow: 
			/* window is already active; change its  size */
			/* make sure it's for mine */
/*		    	if (whichWindow == myWindow)
				resize (myWindow, &event->where); */
		    	break;
		case inGoAway: 
			/* we don't have a GoAway region */
			if (TrackGoAway(whichWindow, &event->where))
				ExitToShell();
			break;
	}
}

/*
 * activate event handling
 *
*/
void
doActivate(event)
EventRecord *event;
{
	/* if for our window, set port as nec. */
	if ((WindowPtr)(event->message) != myWindow) 
		return;
	/* my window */

/*	DrawGrowIcon (myWindow); */
	/* redraw grow icon to reflect new state */

	if (event->modifiers & 1)	/* odd means an activate event */
		SetPort (myWindow);
	else				/* act. evt: work in our own port */

		SetPort (ScreenPort);
		/* deactivate evt: our port is gone; keep port from dangling */
}

/*
 * update event handling
 *
*/
void
doUpdate(event)
EventRecord *event;
{
	if ((WindowPtr) (event->message) == myWindow)
		UpdateWindow (myWindow);  /* redraw the window contents */
}

/*
 * autokey event handling 
*/
/* if command key, pass the char to MenuKey */
void
doAutoKey(event)
EventRecord *event;
{
    	if ((event->modifiers & cmdKey) != 0) 
		doCommand(MenuKey((char)(event->message&CHARCODEMASK)));
}




/*
  body of Skel
*/


main()
{
	SetUp();
	FlushEvents(everyEvent, 0);	/* discard leftover events */
	while (TRUE)
		DoEvent();
}
