/***************************************************************************/
/*                                                                         */
/*                               ErrDialog.c                               */
/*                                                                         */
/*              Contains code for ErrDialog, OSError & ErrMessage          */
/*                   For use when debugging application code               */
/*                                                                         */
/*              10/2/85   DJ Burnard                                       */
/*                                                                         */
/*              10/27/85 - DJB Put error strings into STR# resource        */
/*              11/30/85 - DJB Changed to an Alert!                        */
/*              05/03/86 - DJB Ported to MPW                               */
/*                                                                         */
/***************************************************************************/

/* include MPW C header files */
#include <types.h>
#include <strings.h>
#include <memory.h>
#include <quickdraw.h>
#include <windows.h>
#include <events.h>
#include <dialogs.h>
#include <resources.h>

/* #include <corrections.h> */
/* #include <mystring.h>    */

#define	TRUE	1
#define	FALSE	0
#define	NIL	  	0

/* constant for error dialog kept in the resource fork*/
#define errDialogID      9999
#define errStringID      9999
#define myLastErr        -120
#define noErr				0


      
Str255  *Num2String(theNum, theStr)
  long 	   theNum;
  Str255   *theStr;
  /* This is a minor variation on the Binary-Decimal Conversion
     Package of the same name. Here we have made the procedure
     into a function returning a pointer to the string. */
{
	NumToString(theNum, theStr);
	return theStr;
}

long   String2Num(theStr)
  Str255   *theStr;
  /* This is a minor variation on the Binary-Decimal Conversion
     Package of the same name. Here we have made the procedure
     into a function returning the long number. */
{
	long	theNum;
	
	StringToNum(theStr, &theNum);
	return theNum;
}


/*
 * ErrMessage:                          djb 10/27/85
 *
 *    Retrieve a message describing Mac OS Errors 
 *    Expects the following parameters:
 *
 *      theOSErr - an OSErr - error code returned by OS routine
 *
 *    Message returned is in C format
 */
 
char *ErrMessage(theOSErr, mesg)
   OSErr   theOSErr;
   char   *mesg;
{
   Str255   strtmp;
   
	if(theOSErr == noErr) {
		strcpy(mesg, "noErr: False Alarm");
	} else if(theOSErr > noErr) {
		/* positive error code */
		strcpy(mesg, Num2String(theOSErr, &strtmp));
		strcat(mesg, " = Illegal Error Code > 0");
	} else if((theOSErr < noErr) && (theOSErr > myLastErr)) {
		/* get error message from STR# resource */
		GetIndString(mesg, errStringID, -theOSErr);
	} else {
		/* an error code below the last one in STR# resource */
		strcpy(mesg, Num2String(theOSErr, &strtmp));
		strcat(mesg, ": Error not included in STR# resource");
	}
	return (mesg);
}


/*
 * OSError:                          djb 10/02/85
 *
 *    display an error message for system errors. 
 *    Expects the following parameters:
 *
 *      procName - C string (usually contains name of 
 *                      routine where error occurred)
 *      errNum   - error code returned by system (OSErr)
 *      msg      - C string (arbitrary error message)
 */
 
OSError(procName, errNum, msg)                              
   char    *procName, *msg;
   OSErr   errNum;
{
   Str255   errMsg, trapMsg;
   
   /* paste together trap message */
   strcpy(&trapMsg, procName);
   strcat(&trapMsg, " reports an error");
   
   ErrDialog(&trapMsg, ErrMessage(errNum, &errMsg), msg);
}


/*
 * ErrDialog:                          djb 10/02/85
 *
 *    Alert programmer to an error situation 
 *    Expects the following parameters:
 *
 *      str1, str2, str3 - C strings (arbitrary error messages 
 *                           to the programmer)
 */
 
ErrDialog(str1, str2, str3)
	char   *str1;
	char   *str2;
	char   *str3;
{
	char          *err = "Programmer Error:";
   
	ParamText(err, str1, str2, str3);
	StopAlert(errDialogID, NIL);
}
