/* 
*	Sound Demonstration Program
*	written for the Boston MacWorld Expo
*	using the Macintosh Programmer's Workshop
*
*	 1986 Fred A. Huxham
*
*--------------------------------------
*	This is the Main code
*--------------------------------------
*
*/


/* Macintosh Include Files */
#include <types.h>
#include <resources.h>
#include <quickdraw.h>
#include <fonts.h>
#include <events.h>
#include <windows.h>
#include <controls.h>
#include <dialogs.h>
#include <segload.h>
#include <files.h>
#include <retrace.h>

/* Constant Values */
#define	TRUE		1
#define	FALSE		0
#define	NIL			0
#define MiddleC		25
#define BuffSize	63

/*------------------------------------------------------------------*/
/*	Our own definitions of the SquareWave Sound Data Structures		*/
/*------------------------------------------------------------------*/
#define swMode (-1)

typedef struct Tone {
	short	count;
	short	amplitude;
	short	duration;
} Tone;

typedef Tone Tones[BuffSize];

typedef struct SWSynthRec {
	short	mode;
	Tones	triplets;
} SWSynthRec, *SWSynthPtr;

/* Global Variables */
short		Sound, theNote, **theNotes;
VBLTask		SoundTask;
IOParam		*SndParamPtr;
SWSynthPtr	SqWvPtr;

/* External Functions */
void	VBLGlue();

/*==================================================================*/
/*	Our restart procedure for InitDialogs							*/
/*==================================================================*/
void	reStartProc()
{
   ExitToShell();
}


/*==================================================================*/
/*	Our Vertical Blanking Interrupt routine							*/
/*==================================================================*/
void	mySoundVBL()
{
	SoundTask.vblCount = 1;		/* Reset our task counter */

	if(Sound) {
		if(SndParamPtr->ioActCount >= 370) SndParamPtr->ioActCount = 0;
	}
}


/*==================================================================*/
/*	This routine sets up the Square Wave Synthesizer buffer			*/
/*==================================================================*/
void	SetUpSqWv()
{
	short	i;

	(*SqWvPtr).mode = swMode;
	for(i = 0; i < BuffSize - 1; i++) {
		(*SqWvPtr).triplets[i].count = 2994;
		(*SqWvPtr).triplets[i].amplitude = 200;
		(*SqWvPtr).triplets[i].duration = 1;
	}
	(*SqWvPtr).triplets[BuffSize - 1].count = 0;
	(*SqWvPtr).triplets[BuffSize - 1].amplitude = 0;
	(*SqWvPtr).triplets[BuffSize - 1].duration = 0;

	SndParamPtr->ioBuffer = (Ptr)SqWvPtr;
	SndParamPtr->ioReqCount = (sizeof(SWSynthRec));
}


/*==================================================================*/
/*	This routine starts the sound									*/
/*==================================================================*/
void	StartOurSound()
{
	short	result;

	result = PBWrite(SndParamPtr, TRUE);
	if(result) OSError("PBWrite", result, "");
	Sound = TRUE;
}


/*==================================================================*/
/*	This routine stops the sound									*/
/*==================================================================*/
void	StopOurSound()
{
	short	result;

	result = PBKillIO(SndParamPtr, TRUE);
	if(result) OSError("PBKillIO", result, "");
	Sound = FALSE;
}


/*==================================================================*/
/*	This routine changes the note to play							*/
/*==================================================================*/
ChangeNote(whichNote)
	short	whichNote;
{
	short	newCount, i;
	
	newCount = (*theNotes)[whichNote - 1];

	(*SqWvPtr).mode = swMode;
	for(i = 0; i < BuffSize - 1; i++) {
		(*SqWvPtr).triplets[i].count = newCount;
		(*SqWvPtr).triplets[i].amplitude = 200;
		(*SqWvPtr).triplets[i].duration = 1;
	}
	(*SqWvPtr).triplets[BuffSize - 1].count = 0;
	(*SqWvPtr).triplets[BuffSize - 1].amplitude = 0;
	(*SqWvPtr).triplets[BuffSize - 1].duration = 0;
}


/*==================================================================*/
/*	DoModeless takes a dialog item and a modeless dialog pointer	*/
/*	as arguments, and responds to the user's actions appropriately	*/
/*==================================================================*/
void	DoModeless(whichItem,whichDialog)
short		whichItem;
DialogPtr	whichDialog;
{
	short			itemType;
	ControlHandle	item;
	Rect			box;

	if(whichItem >= 1 && whichItem <= 60) {
		GetDItem(whichDialog, theNote, &itemType, &item, &box);
		SetCtlValue(item, FALSE);
		GetDItem(whichDialog, whichItem, &itemType, &item, &box);
		SetCtlValue(item, TRUE);
		theNote = whichItem;
		ChangeNote(whichItem);
	}
}


/*==================================================================*/
/*	DoWindowStuff takes care of mouse-downs in the drag and goAway	*/
/*	regions of the modeless dialog									*/
/*==================================================================*/
void	DoWindowStuff(theEvent,whichWindow,windowcode)
EventRecord	*theEvent;
WindowPtr 	whichWindow;
short		windowcode;
{
	short	result;
	Rect	screenRect;
	
	SetRect(&screenRect,10,30,502,332);
	switch(windowcode) {
		case inDrag:
			DragWindow(whichWindow,&theEvent->where,&screenRect);
			break;
		case inGoAway:
			if(TrackGoAway(whichWindow,&theEvent->where)) {
				result = VRemove(&SoundTask);
				if(result) OSError("VRemove", result, "");
				StopOurSound();
				ExitToShell();
			}
			break;
	}
}


/*==================================================================*/
/*	Here's our main event loop										*/
/*==================================================================*/
main()
{
    extern struct qd qd;

	EventRecord		theEvent;
	DialogPtr		modeless,whichDialog;
	short			whichItem,windowcode, itemType, i, result;
	WindowPtr		whichWindow;
	Rect			box;
	ControlHandle	item;

	/* The Standard Initializations */
    InitGraf(&qd.thePort);
    InitFonts();
    FlushEvents(everyEvent, 0);
    InitWindows();
	InitDialogs(reStartProc);
	InitCursor();

	/* Initialize our global variables */
	Sound = FALSE;
	theNote = MiddleC;

	/* Allocate our Pointers */
	SqWvPtr = (SWSynthPtr)NewPtr(sizeof(SWSynthRec));
	SndParamPtr = (IOParam *)NewPtr(sizeof(IOParam));
	
	/* Load the Notes resource */
	theNotes = (short **)GetResource('Note', 8888);
	if(ResError()) OSError("GetResource", ResError(), "in main()");
	DetachResource(theNotes);
	if(ResError()) OSError("DetachResource", ResError(), "in main()");


	/* Set up the fields of the VBLTask */
	SoundTask.qType = 1;					/* 1 = vType */
	SoundTask.vblAddr = VBLGlue;
	SoundTask.vblCount = 1;
	SoundTask.vblPhase = 0;

	/* Set up the Sound Parameter block */
	SndParamPtr->ioCompletion = NIL;
	SndParamPtr->ioRefNum = (-4);

	/* Fill in our SquareWave Pointer */
	SetUpSqWv();

	/* put up the modeless dialog with middle C checked */
	modeless = GetNewDialog(1000,NIL,(WindowPtr)-1);
	GetDItem(modeless, theNote, &itemType, &item, &box);
	SetCtlValue(item, TRUE);
	ShowWindow(modeless);

	/* Install the VBLTask and then start the sound */
	result = VInstall(&SoundTask);
	if(result) OSError("VInstall", result, "");
	StartOurSound();

	/* And here's the loop */
	while (TRUE)   {
		if(GetNextEvent(everyEvent,&theEvent)) {
			if(IsDialogEvent(&theEvent)) {
				if(DialogSelect(&theEvent,&whichDialog,&whichItem)) {
					DoModeless(whichItem,whichDialog);
				}
			}
			switch (theEvent.what) {
				case mouseDown:
					windowcode = FindWindow(&theEvent.where,&whichWindow);
					DoWindowStuff(&theEvent,whichWindow,windowcode);
					break;
			}
		}
	}
}
